#!/usr/bin/env python3
"""
Compare various guidance law (ascent y)
"""

import sympy as sp
from matplotlib import pyplot as plt

from utils.control_laws import get_acc_cmd_1, get_acc_cmd_2, get_acc_cmd_3, get_acc_cmd_4, get_acc_cmd_5, get_acc_cmd_6
from utils.plot import MATPLOTLIB_FIGSIZE
from utils.sympy import symbols_get, trajectory_compute


def print_acc_cmd_info(s, vals, acc_cmd):
    tgosol = sp.solve([sp.Eq(acc_cmd, s['af'])], [s['tgo']])
    print('\t* tgosol', tgosol)

    a = acc_cmd \
        .subs(s['af'], vals['AF']).subs(s['vf'], vals['VF']).subs(s['pf'], vals['PF']) \
        .subs(s['v0'], vals['V0']).subs(s['p0'], vals['P0']) \
        .subs(s['tgo'], vals['TGO'])
    print('\t* a', float(a))


def plot(cmds, legends, s, vals):
    TGO = vals['TGO']

    print('plot...')

    plt.figure(figsize=MATPLOTLIB_FIGSIZE)

    plot_as = plt.subplot(3, 1, 1)
    plt.xlim(0, TGO)

    plot_vs = plt.subplot(3, 1, 2)
    plt.xlim(0, TGO)

    plot_ps = plt.subplot(3, 1, 3)
    plt.xlim(0, TGO)

    for p in (
        plot_as,
        plot_vs,
        plot_ps,
    ):
        p.hlines(0, 0, TGO, color='gray')

    for cmd in cmds:
        print('\tcmd', cmd)
        print_acc_cmd_info(s, vals, cmd)

        accs, vels, poss = trajectory_compute(s, vals, cmd)

        print('\t* last a', accs[-1])
        print('\t* last v', vels[-1])
        print('\t* last p', poss[-1])

        plot_as.plot(accs)
        plot_vs.plot(vels)
        plot_ps.plot(poss)

    plot_as.legend(legends)

    plot_as.set_ylabel('acc_y (m/s**2)')
    plot_vs.set_ylabel('vel_y (m/s)')
    plot_ps.set_ylabel('pos_y (m)')

    print('Saving plot to output/2a-cmp.png')
    plt.tight_layout()
    plt.savefig('output/2a-cmp.png')


def main():

    # Ascent
    # vals = {
    #     'V0': 0,
    #     'P0': 0,
    #     'AF': 0,
    #     'VF': 0,
    #     'PF': 15_000,
    #     'TGO': 600,
    # }

    # Descent
    vals = {
        'V0': 0,
        'P0': 15_000,
        'AF': 0,
        'VF': 0,
        'PF': 0,
        'TGO': 600,
    }

    s = symbols_get()

    acc_cmd1 = get_acc_cmd_1(s, vals)
    acc_cmd2 = get_acc_cmd_2(s, vals)
    acc_cmd3 = get_acc_cmd_3(s, vals)
    acc_cmd4 = get_acc_cmd_4(s, vals)
    acc_cmd5 = get_acc_cmd_5(s, vals)
    acc_cmd6 = get_acc_cmd_6(s, vals)

    cmds = (
        acc_cmd1,
        acc_cmd3,
        acc_cmd5,
        acc_cmd6,
    )
    legends = [
        'constant (acc = k1)',
        'linear (acc = k1*t + k2)',
        'quadratic (acc = k1*t**2 + k2*t + k3)',
        'cubic (t**3)',
    ]

    plot(cmds, legends, s, vals)
    plt.show()


if __name__ == '__main__':
    main()
